@@ -32,7 +32,8 @@
 
 #include "src/internal.h"
 
-void bytefn(dav1d_cdef_brow)(Dav1dFrameContext *f, pixel *const p[3],
-                             const Av1Filter *lflvl, int by_start, int by_end);
+void bytefn(dav1d_cdef_brow)(Dav1dTaskContext *tc, pixel *const p[3],
+                             const Av1Filter *lflvl, int by_start, int by_end,
+                             int sbrow_start, int sby);
 
 #endif /* DAV1D_SRC_CDEF_APPLY_H */
