@@ -1,5 +1,5 @@
 /*
- * Copyright © 2018, VideoLAN and dav1d authors
+ * Copyright © 2018-2021, VideoLAN and dav1d authors
  * Copyright © 2018, Two Orioles, LLC
  * All rights reserved.
  *
@@ -34,16 +34,16 @@
 #include "src/levels.h"
 
 typedef struct Av1FilterLUT {
-    uint8_t e[64];
-    uint8_t i[64];
-    uint64_t sharp[2];
+    ALIGN(uint8_t e[64], 16);
+    ALIGN(uint8_t i[64], 16);
+    ALIGN(uint64_t sharp[2], 16);
 } Av1FilterLUT;
 
 typedef struct Av1RestorationUnit {
+    /* SGR: type = DAV1D_RESTORATION_SGRPROJ + sgr_idx */
     uint8_t /* enum Dav1dRestorationType */ type;
     int8_t filter_h[3];
     int8_t filter_v[3];
-    uint8_t sgr_idx;
     int8_t sgr_weights[2];
 } Av1RestorationUnit;
 
