@@ -244,7 +244,19 @@ static void loop_filter_v_sb128uv_c(pixel *dst, const ptrdiff_t stride,
     }
 }
 
-void bitfn(dav1d_loop_filter_dsp_init)(Dav1dLoopFilterDSPContext *const c) {
+#if HAVE_ASM
+#if ARCH_AARCH64 || ARCH_ARM
+#include "src/arm/loopfilter.h"
+#elif ARCH_LOONGARCH64
+#include "src/loongarch/loopfilter.h"
+#elif ARCH_PPC64LE
+#include "src/ppc/loopfilter.h"
+#elif ARCH_X86
+#include "src/x86/loopfilter.h"
+#endif
+#endif
+
+COLD void bitfn(dav1d_loop_filter_dsp_init)(Dav1dLoopFilterDSPContext *const c) {
     c->loop_filter_sb[0][0] = loop_filter_h_sb128y_c;
     c->loop_filter_sb[0][1] = loop_filter_v_sb128y_c;
     c->loop_filter_sb[1][0] = loop_filter_h_sb128uv_c;
@@ -252,9 +264,13 @@ void bitfn(dav1d_loop_filter_dsp_init)(Dav1dLoopFilterDSPContext *const c) {
 
 #if HAVE_ASM
 #if ARCH_AARCH64 || ARCH_ARM
-    bitfn(dav1d_loop_filter_dsp_init_arm)(c);
+    loop_filter_dsp_init_arm(c);
+#elif ARCH_LOONGARCH64
+    loop_filter_dsp_init_loongarch(c);
+#elif ARCH_PPC64LE
+    loop_filter_dsp_init_ppc(c);
 #elif ARCH_X86
-    bitfn(dav1d_loop_filter_dsp_init_x86)(c);
+    loop_filter_dsp_init_x86(c);
 #endif
 #endif
 }
