@@ -27,7 +27,7 @@
 
 #include DAV1D_TEST_HEADER
 
-int main()
+int main(void)
 {
     return 0;
 }
