@@ -40,7 +40,7 @@
 
 // expects ivf input
 
-int main(const int argc, char *const *const argv) {
+int main(int argc, char *argv[]) {
     int ret = -1;
     FILE *f = NULL;
     int64_t fsize;
@@ -48,6 +48,10 @@ int main(const int argc, char *const *const argv) {
     uint8_t *data = NULL;
     size_t size = 0;
 
+    if (LLVMFuzzerInitialize(&argc, &argv)) {
+        return 1;
+    }
+
     if (argc != 2) {
         fprintf(stdout, "Usage:\n%s fuzzing_testcase.ivf\n", argv[0]);
         return -1;
