@@ -35,7 +35,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifdef HAVE_UNISTD_H
+#if HAVE_UNISTD_H
 # include <unistd.h>
 #endif
 
@@ -100,11 +100,11 @@ static const struct option long_opts[] = {
 #endif
 
 #if ARCH_AARCH64 || ARCH_ARM
-#define ALLOWED_CPU_MASKS " or 'neon'"
+#define ALLOWED_CPU_MASKS ", 'neon', 'dotprod' or 'i8mm'"
 #elif ARCH_LOONGARCH
 #define ALLOWED_CPU_MASKS ", 'lsx' or 'lasx'"
 #elif ARCH_PPC64LE
-#define ALLOWED_CPU_MASKS " or 'vsx'"
+#define ALLOWED_CPU_MASKS ", 'vsx' or 'pwr9'"
 #elif ARCH_RISCV
 #define ALLOWED_CPU_MASKS " or 'rvv'"
 #elif ARCH_X86
@@ -215,22 +215,43 @@ enum CpuMask {
     X86_CPU_MASK_AVX2      = DAV1D_X86_CPU_FLAG_AVX2      | X86_CPU_MASK_SSE41,
     X86_CPU_MASK_AVX512ICL = DAV1D_X86_CPU_FLAG_AVX512ICL | X86_CPU_MASK_AVX2,
 };
+#elif ARCH_AARCH64 || ARCH_ARM
+enum CpuMask {
+    ARM_CPU_MASK_NEON      = DAV1D_ARM_CPU_FLAG_NEON,
+    ARM_CPU_MASK_DOTPROD   = DAV1D_ARM_CPU_FLAG_DOTPROD | ARM_CPU_MASK_NEON,
+    ARM_CPU_MASK_I8MM      = DAV1D_ARM_CPU_FLAG_I8MM    | ARM_CPU_MASK_DOTPROD,
+#if ARCH_AARCH64
+    // SVE doesn't imply DOTPROD or I8MM.
+    ARM_CPU_MASK_SVE       = DAV1D_ARM_CPU_FLAG_SVE     | ARM_CPU_MASK_NEON,
+    // SVE2 implies DOTPROD, but not I8MM.
+    ARM_CPU_MASK_SVE2      = DAV1D_ARM_CPU_FLAG_SVE2    | ARM_CPU_MASK_SVE | ARM_CPU_MASK_DOTPROD,
 #endif
+};
+#endif
+
+#if ARCH_PPC64LE
+enum CpuMask {
+    PPC_CPU_MASK_VSX       = DAV1D_PPC_CPU_FLAG_VSX,
+    PPC_CPU_MASK_PWR9      = DAV1D_PPC_CPU_FLAG_VSX | DAV1D_PPC_CPU_FLAG_PWR9,
+};
+#endif
+
 
 static const EnumParseTable cpu_mask_tbl[] = {
 #if ARCH_AARCH64 || ARCH_ARM
-    { "neon",    DAV1D_ARM_CPU_FLAG_NEON },
-    { "dotprod", DAV1D_ARM_CPU_FLAG_DOTPROD },
-    { "i8mm",    DAV1D_ARM_CPU_FLAG_I8MM },
+    { "neon",    ARM_CPU_MASK_NEON },
+    { "dotprod", ARM_CPU_MASK_DOTPROD },
+    { "i8mm",    ARM_CPU_MASK_I8MM },
 #if ARCH_AARCH64
-    { "sve",     DAV1D_ARM_CPU_FLAG_SVE },
-    { "sve2",    DAV1D_ARM_CPU_FLAG_SVE2 },
+    { "sve",     ARM_CPU_MASK_SVE },
+    { "sve2",    ARM_CPU_MASK_SVE2 },
 #endif /* ARCH_AARCH64 */
 #elif ARCH_LOONGARCH
     { "lsx", DAV1D_LOONGARCH_CPU_FLAG_LSX },
     { "lasx", DAV1D_LOONGARCH_CPU_FLAG_LASX },
 #elif ARCH_PPC64LE
-    { "vsx", DAV1D_PPC_CPU_FLAG_VSX },
+    { "vsx",  PPC_CPU_MASK_VSX },
+    { "pwr9", PPC_CPU_MASK_PWR9 },
 #elif ARCH_RISCV
     { "rvv", DAV1D_RISCV_CPU_FLAG_V },
 #elif ARCH_X86
